#!BPY
""" Registration info for Blender menus
Name: 'ph_SymmetriseMesh'
Blender: 234
Group: 'Mesh'
Tooltip: 'Symmetrise mesh from one side to the other side.'
"""

#### !<>

########################################################
#													   #
#				 ph_SymmetriseMesh.py				   #
#													   #
#		  Cree par coolphil@worldonline.fr			   #
#				   le 12/08/2004					   #
#													   #
#		 Derniere mise a jour: le 28/08/2004		   #
#													   #
#	  Symmetrise le mesh d'un cote vers l'autre.	   #
#													   #
#			Prend en compte la geometrie			   #
#			et tous les autres attributs.			   #
#													   #
# http://hp.home.chez.tiscali.fr/re_illusion/index.htm #
#													   #
########################################################


import Blender, webbrowser
from Blender.sys import *
from Blender import Object, NMesh, Window
from Blender.Draw import *
from Blender.BGL import *
from math import *

######################

tolerence = 0.001
side = Create(0)
sideAffichage = "X to -X"

######################

#### Global Variables #############

modemesh = 0
warn = False
start = 0.0
end = 0.0
DelayWanted = 0.5

EVENT_EXIT = 1
EVENT_ACTION = 2

###### Quelques couleurs #######
white = (1.0,1.0,1.0)
black = (0.0,0.0,0.0)
blue = (.71,.71,.76)
dgray = (.4,.4,.4)
red = (.7,.0,.3)
green = (.70,.75,.70)
back = (.7,.7,.7)
################################

WindowSize = Buffer(GL_FLOAT, 4)
glGetFloatv(GL_SCISSOR_BOX, WindowSize)
WindowSize = WindowSize.list

H_Col = blue
Border = (H_Col[0]-.2,H_Col[1]-.2,H_Col[2]-.2)
Glanz = (H_Col[0]+.2,H_Col[1]+.2,H_Col[2]+.2)


#### Variables Mouse_Over ####
Col_M_Over = (0.0,0.0,0.0)
hp_x = 23
hp_y = 3
Chk_M_Over = 3
##############################

BWidth = 230
BHeight = 160

####### Variables pour le deplacement #######
mouse_x = 0									#
mouse_y = 0									#
											#
barStartx = 1								#
barStarty = BHeight - 35					#
barLenx = 195								#
barLeny = 35								#
											#
L_M = 0										#
v_x = 0										#
v_y = 0										#
Bposx = 0									#
Bposy = 0									#
lastBposx = 0								#
lastBposy = 0								#
#############################################

################
def GL_Bx(vl=0):
################
	return (Bposx + vl)

################
def GL_By(vl=0):
################
	return (Bposy + vl)

#############################
def Ligne(y,xs=20,xe=BWidth):
#############################
	glRecti(xs,y,xe,y+1)

######################################
def Ligne3D(y,long,col,flat=20,xs=20):
######################################
	t= flat/100.0
	glColor3d(col[0]+t,col[1]+t,col[2]+t)
	Ligne(y,xs,xs+long)
	glColor3d(col[0]-t,col[1]-t,col[2]-t)
	Ligne(y+1,xs,xs+long)
	
#############################################################################
def Degrade(y,height,col,flat=10,xs=20,xe=BWidth,half=0,bizS=0,bizE=0):
#############################################################################
	if not((bizS == 0) or (bizS == -1) or (bizS == 1)):
		bizS = 0
 	if not((bizE == 0) or (bizE == -1) or (bizE == 1)):
		bizE = 0


	cC = 0
	cT = 1+half
	for cl in range(0, height, (1 - 2 * (height < 0))):
		incS = xs + ( height * cl/height * bizS)
		incE = xe + ( height * cl/height * bizE)

		cC = -cos(cl/float(height)*pi/(cT+(cT==0)))*flat/100
		glColor3d(col[0]+cC,col[1]+cC,col[2]+cC)
		Ligne(y+cl,incS,incE)


##################################################
def GL_Dot(x=10, y=10, col=(.71,.71,.76), flat=2):
##################################################
	DotList = ((-.05,-.1,-.1),(.05,.0,-.1),(.1,.05,-.05))
	glBegin(GL_POINTS)
	for ze in range(3):
		for sp in range(3):
			cC = DotList[ze][sp]*flat
			glColor3d(col[0]+cC,col[1]+cC,col[2]+cC)
			glVertex2i(x+sp,y+ze)
	glEnd()

##################################################################################
def GL_DotLine(x=10, y=10, long = 10, col=(.71,.71,.76), direction = "y", flat=5):
##################################################################################

	if direction == "x":
		for Dot_X in range(x, x+long, 10):
			GL_Dot(Dot_X, y, col, flat)

	else:
		for Dot_Y in range(y, y+long, 10):
			GL_Dot(x, Dot_Y, col, flat)

	return 0

###########################################################################################
def Relief_Text(x,y,t, colfrt=(1.0, 1.0, 1.0), colback=(0.0, 0.0, 0.0), fontsize ='normal'):
###########################################################################################


	if fontsize not in ('normal', 'small', 'tiny', 'n', 's', 't', 'N', 'S', 'T'):
		fontsize = 'normal'
	else:
		fontsize = 'normal' * (fontsize in ('normal', 'n', 'N')) + ('small' *(fontsize in ('small', 's', 'S'))) + ('tiny' *(fontsize in ('tiny', 't', 'T')))

	if fontsize == 'normal':
		apply(glColor3f, colback)
		glRasterPos2f(x+1, y+3)
		Text(t,fontsize)
		glRasterPos2f(x+1, y+4)
		Text(t,fontsize)
		apply(glColor3f, colfrt)
		glRasterPos2f(x, y+5)
		Text(t,fontsize)
	else:
		apply(glColor3f, colback)
		glRasterPos2f(x+1, y+4)
		Text(t,fontsize)
		apply(glColor3f, colfrt)
		glRasterPos2f(x, y+5)
		Text(t,fontsize)

	return 0

#########################################################################################
#########################################################################################


################
def editbegin():
################
	global edit_mode, modemesh

	try:
		edit_mode = Window.EditMode()
		if edit_mode: Window.EditMode(0); modemesh = 1
	except:
		print "=============================="
		print "'EditMode()' INCOMPATIBILITY !"
	return 0

##############
def editend():
##############
	global modemesh

	try:
		if edit_mode: Window.EditMode(1)
	except:
		print "=============================="
		print "'EditMode()' INCOMPATIBILITY !"

	modemesh = 0

	return 0

###############
def info_Obj(Ob=[]):
###############

	Obtype = 0
	mename = []
	me = []
	calque = 1

	if not Ob:
		if not Object.GetSelected():
			return Ob,Obtype,mename,me
		else:
			Ob = Object.GetSelected()[0]
			if (Ob.getType() == 'Mesh'):

				Obtype = 1
				calque = Ob.Layer                    # Le "layer" de l'object selectionne
				mename = Ob.getData().name
				me = Ob.getData()
	else:
		if (Ob.getType() == 'Mesh'):

			Obtype = 1
			calque = Ob.Layer                    # Le "layer" de l'object selectionne
			mename = Ob.getData().name
			me = Ob.getData()

	return Ob,Obtype,mename,me


#######################
#### Fonction reduce ##
def reduce(list):     #
#######################

	list.sort()
	list.reverse()

	lenlist = len(list) - 1
	if lenlist > 0:
		compt = 0

		while compt < lenlist:

			if list[compt] == list[compt + 1]:
				list.remove(list[compt])
				lenlist -= 1

			else:
				compt += 1

	list.reverse()

	return list

###############
def ph_Timer():
###############
	global warn

	Draw()

	TheTime = 0.0

	while (TheTime < DelayWanted):
		end = time()
		TheTime = end - start

	warn = False
	Draw()

	return 0

###############
def DelFaces():
###############
	global tolerence, warn, start, mesh

	object,Objtype,meshname,mesh = info_Obj()

	if Objtype:
		if len(mesh.faces) > 1:

			rescue = object.getData()

			######################
			#Netoyage points milieux
			######################

			for vertices in mesh.verts:
				if (vertices.co[0] >= -tolerence and vertices.co[0] <= tolerence): ######## -0.001 a 0.001 est l'interval de tolerence
					vertices.co[0] = 0.00

			######################


			FList = []
			VertList = []

			if side.val:
				comptFaces = 0
				for f in mesh.faces:

					check = 0
					Flen = len(f.v)
					for vertices in f.v:
						if vertices.co[0] > 0.00:
							check += 1
					if check == Flen:
						FList.append(f)
						for v in f.v:
							VertList.append(v)
					else:
						check = 0
						for vertices in f.v:
							if vertices.co[0] >= 0.00:
								check += 1
						if check == Flen:
							FList.append(f)
					comptFaces += 1
				reduce(VertList)

			else:
				comptFaces = 0
				for f in mesh.faces:

					check = 0
					Flen = len(f.v)
					for vertices in f.v:
						if vertices.co[0] < 0.00:
							check += 1
					if check == Flen:
						FList.append(f)
						for v in f.v:
							VertList.append(v)
					else:
						check = 0
						for vertices in f.v:
							if vertices.co[0] <= 0.00:
								check += 1
						if check == Flen:
							FList.append(f)
					comptFaces += 1
				reduce(VertList)

			########################
			# Deleting faces & vertices
			####################################
			FList = map(deleteFaces,FList)
			VertList = map(deleteVerts,VertList)
			####################################

			if 1 == 1:
				try:
					Symmetrise(mesh)

					mesh.update()
					object.makeDisplayList()
				except:
					warn = True
					start = time()
					print "\a\r" ## >>>>> Optionnel beep <<<<< ##
					print "#########################"
					print "#       WARNING:        #"
					print "# No symmetrical mesh ! #"
					print "#########################"
					rescue.update()
					object.makeDisplayList()
###################
def deleteFaces(x):
###################
	del mesh.faces[mesh.faces.index(x)]

###################
def deleteVerts(x):
###################
	del mesh.verts[mesh.verts.index(x)]


###################
def Symmetrise(me):
###################

	NewV = {}

	########## Duplication des faces et vertex #########
	lenf = len(me.faces)

	for findx in range(lenf):
		f = me.faces[findx]

		NewF = NMesh.Face()

		NewVcol = []

		for vindx in range(len(f.v)):
			v = f.v[vindx]
			try:
				if vindx == 0:
					NewF.v.insert(0, NewV[v])
					try:
						NewVcol.insert(0, f.col[vindx])
					except:
						pass

				else:
					NewF.v.insert(1, NewV[v])
					try:
						NewVcol.insert(1, f.col[vindx])
					except:
						pass

			except:
				if v.co[0] == 0.00:
					NewV[v] = v

				else: #### ajout d'un nouveau vertice ####
					me.verts.append(NMesh.Vert(-v.co[0], v.co[1], v.co[2]))
					NewV[v] = me.verts[-1]
					if me.verts[me.verts.index(v)].sel:
						me.verts[-1].sel = 1

				if vindx == 0:
					NewF.v.insert(0, NewV[v])
					try:
						NewVcol.insert(0, f.col[vindx])
					except:
						pass
				else:
					NewF.v.insert(1, NewV[v])
					try:
						NewVcol.insert(1, f.col[vindx])
					except:
						pass



		me.faces.append(NewF)
		me.faces[-1].smooth = f.smooth

		###### Copy Attributs #####
		N_f = me.faces[-1]
		CopyFaceAttribut(N_f, f, me, NewVcol)
		###########################


	return 0

###################################
def CopyFaceAttribut(New_F, Ori_F, me,listVcol):
###################################

		tp = []

		try:
			if len(New_F.v) == 4:
				tp.append(Ori_F.uv[0])
				tp.append(Ori_F.uv[3])
				tp.append(Ori_F.uv[2])
				tp.append(Ori_F.uv[1])

			elif len(New_F.v) == 3:
				tp.append(Ori_F.uv[0])
				tp.append(Ori_F.uv[2])
				tp.append(Ori_F.uv[1])

			New_F.uv = tp
		except:
			pass
		New_F.flag = Ori_F.flag

		##### VertexColors #####
		for vindx in range(len(New_F.v)):
			v = me.verts[me.verts.index(Ori_F.v[vindx])]

			try:
				New_F.col.append(NMesh.Col(listVcol[vindx].r, listVcol[vindx].g, listVcol[vindx].b, listVcol[vindx].a))
			except:
				pass


		##### Les autres attributs #####
		if Ori_F.image:
			New_F.image = Ori_F.image
		New_F.mode = Ori_F.mode
		New_F.transp = Ori_F.transp
		New_F.materialIndex = Ori_F.materialIndex

		return 0

###########
def draw():
###########
	global side, sideAffichage
	global WindowSize

	color1 = (0.99, 0.99, 0.65)
	color2 = (0.567, 0.476, 0.365)
	color2Clear = (0.567, 0.476, 0.365, 1)
	color3 = (1.0, 1.0, 0.79)
	color4 = (0.265, 0.41, 0.4)
	color4Clear = (0.265, 0.41, 0.4, 1)
	color5 = (0.4, 0.472, 0.472)
	color6 = (0.5, 0.25, 0.25)
	color7 = (0.265, 0.4, 0.3)
	colorblack = (0.0, 0.0, 0.0)
	colorwhite = (1.0, 1.0, 1.0)
	colorred = (1.0, 0.0, 0.0)
	colorcyan = (0.0, 1.0, 1.0)
	colorgrey1 = (0.8, 0.8, 0.8)

	WindowSize = Buffer(GL_FLOAT, 4)
	glGetFloatv(GL_SCISSOR_BOX, WindowSize)
	WindowSize = WindowSize.list
#	print WindowSize

	apply(glClearColor, color4Clear)
	glClear(GL_COLOR_BUFFER_BIT)

########

	Degrade(GL_By(- 1), BHeight - 2, (0.8, 0.8, 0.6), 20,GL_Bx(1),GL_Bx(BWidth), -0.4)

	if warn:
		apply(glColor3f, colorblack)
		glRecti(GL_Bx(1), GL_By(- 1), GL_Bx(BWidth), GL_By(BHeight - 3))
		apply(glColor3f, colorgrey1)
		glRecti(GL_Bx(), GL_By(1), GL_Bx(BWidth - 1), GL_By(BHeight - 2))
		apply(glColor3f, colorred)
		glRecti(GL_Bx(1), GL_By(1), GL_Bx(BWidth - 1), GL_By(BHeight - 3))


	Degrade(GL_By(BHeight -39), 35, (0.85, 0.7, 0.5), 35,GL_Bx(1),GL_Bx(BWidth - 1), 0.00)
	Degrade(GL_By(BHeight -39), 35, (0.85, 0.7, 0.5), 60,GL_Bx(1),GL_Bx(135), 0.44,0,1)
	Relief_Text(GL_Bx(10), GL_By(BHeight - 28),"ph_SymmetriseMesh.py", (0.3,0.3,0.3), (0.962, 0.792, 0.566))


	GL_DotLine(GL_Bx(10), GL_By(10), 90)
	GL_DotLine(GL_Bx(BWidth - 10), GL_By(10), 90)


	Ligne3D(GL_By(20),50,(0.8, 0.8, 0.6),-30,GL_Bx(20))
	Ligne3D(GL_By(BHeight - 50),35,(0.8, 0.8, 0.6),-30,GL_Bx(BWidth - 55))

	Relief_Text(GL_Bx(hp_x), GL_By(hp_y),"http://hp.home.chez.tiscali.fr/re_illusion/index.htm", Col_M_Over, (0.8, 0.8, 0.8), 't')

#########

	Button("Exit", EVENT_EXIT, GL_Bx(BWidth - 50), GL_By(BHeight - 30), 40, 20, "Quit")

	Button("Symmetrise", EVENT_ACTION, GL_Bx(65), GL_By(35), 100, 30, 'Symmetrise mesh from one side to the other side.')

	side = Toggle(sideAffichage,4,GL_Bx(BWidth/2-30), GL_By(BHeight - 80),60,20,side.val,"Side to symmetrise")

#########


####################
def event(evt, val):
####################
	global mouse_x, mouse_y, L_M, L_Mx, L_My, v_x, v_y, barStartx, barStarty, Bposx, Bposy, lastBposx, lastBposy
	global Col_M_Over, Chk_M_Over


	if (evt== QKEY and not val): Exit()

	############################################################################
	################################# Deplacements #############################

	if (evt == MOUSEX): mouse_x = int(val - WindowSize[0])
	if (evt == MOUSEY): mouse_y = int(val - WindowSize[1])

	if (evt == LEFTMOUSE and val) and (mouse_x in range(barStartx,barStartx+barLenx)) and (mouse_y in range(barStarty,barStarty+barLeny)):

		L_M = 1
		L_Mx = mouse_x
		L_My = mouse_y

	elif (evt == LEFTMOUSE and not val):
		L_M = 0
		barStartx += v_x
		barStarty += v_y

		lastBposx = Bposx
		lastBposy = Bposy
		v_x = 0
		v_y = 0

	
	if (evt == MOUSEX) and L_M:
		mouse_x = int(val - WindowSize[0])
		v_x = mouse_x - L_Mx
		Bposx = lastBposx + v_x
		Bposy = lastBposy + v_y
		Redraw()

	if (evt == MOUSEY) and L_M:
		mouse_y = int(val - WindowSize[1])
		v_y = mouse_y - L_My
		Bposx = lastBposx + v_x
		Bposy = lastBposy + v_y
		Redraw()

	if (evt == RIGHTMOUSE) and not L_M: #### Reinitialisation ####
		barStartx = 1
		barStarty = BHeight - 35
		lastBposx = 0
		lastBposy = 0
		Bposx = 0
		Bposy = 0
		v_x = 0
		v_y = 0
		Redraw()
	############################################################################
	############################################################################

#### Mouse_Over ####
	if (mouse_x in range(GL_Bx(hp_x),GL_Bx(hp_x)+185)) and (mouse_y in range(GL_By(hp_y+5),GL_By(hp_y+5)+10)):
		Col_M_Over = (1.0,0.0,0.0)
		Chk_M_Over = 1
		Redraw()
	else:
		Col_M_Over = (0.0,0.0,0.0)
		Chk_M_Over += 1
	if Chk_M_Over == 2: Redraw(); Chk_M_Over += 1

####################


#### Mouse_Over_Pushed ####
	if (evt == LEFTMOUSE and val) and (mouse_x in range(GL_Bx(hp_x),GL_Bx(hp_x)+185)) and (mouse_y in range(GL_By(hp_y+5),GL_By(hp_y+5)+10)):
		webbrowser.open('http://hp.home.chez.tiscali.fr/re_illusion/index.htm')
###########################


################
def bevent(evt):
################
	global sideAffichage

	if (evt== EVENT_EXIT):
		Exit()

	elif (evt== EVENT_ACTION):
		editbegin()
		DelFaces()
		editend()

	elif (evt== 4):
		if side.val:
			sideAffichage = "-X to X"
		else:
			sideAffichage = "X to -X"



#####

	if warn:
		ph_Timer()

	Draw()


#############################
Register(draw, event, bevent)
#############################